@@ -480,11 +480,14 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	char catchname[256];
 	const char *embed = NULL;
 	bool mscmd = false, execflag = false;
+	bool apps = false;
 
 	if(strstr(".bat.cmd", cp))
 		mscmd = true;
 
-	if(strstr(exec_extensions, cp))
+	if(strstr(apps_extensions, cp))
+		apps = true;
+	else if(strstr(exec_extensions, cp))
 	{
 		snprintf(namebuf, sizeof(namebuf), "%s::%s", exec_prefix, name);
 		name = namebuf;
@@ -612,10 +615,11 @@ compile:
 			scrStream->getline(buffer + offset, bufsize - 1 - offset);
 			if(scrStream->eof())
 			{
-				if(embed && !execflag)
+				if((embed) && !execflag)
 				{
 					execflag = true;
 					embed = NULL;
+					apps = false;
 					setString(buffer, 16, "exec");
 				}
 				else if(embed || (!total && !count && !offset))
@@ -720,7 +724,7 @@ first:
 				break;
 			}
 
-			if(!cmds->ripple && !embed)
+			if(!cmds->ripple && !embed && !apps)
   			  if(!stricmp(token, "private") || !stricmp(token,
 "protected") || !stricmp(token, "public") || !stricmp(token, "program")
 || !stricmp(token, "state") || !stricmp(token, "module"))
@@ -756,27 +760,27 @@ repname1:
 			}
 
 
-			if(!stricmp(token, "service") && !cmds->ripple && !embed)
+			if(!stricmp(token, "service") && !cmds->ripple && !embed && !apps)
 			{
 				ftoken = "options";
 				pmode = "public";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "func", 4) && !cmds->ripple && !embed)
+			if(!strnicmp(token, "func", 4) && !cmds->ripple && !embed && !apps)
 			{
 				ftoken = "fconst";
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "proc", 4) && !cmds->ripple && !embed)
+			if(!strnicmp(token, "proc", 4) && !cmds->ripple && !embed && !apps)
 			{
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!stricmp(token, "catch") && !cmds->ripple && !embed)
+			if(!stricmp(token, "catch") && !cmds->ripple && !embed && !apps)
 			{
 				ftoken = "fconst";
 				name = getToken();
@@ -801,7 +805,7 @@ repname1:
 				break;
 			}
 			
-			if(cmds->ripple && *token == '[')
+			if((cmds->ripple || apps) && *token == '[')
 			{
 				name = token + 1;
 				cp = strchr(name, ']');
@@ -873,10 +877,10 @@ repname1:
 		if(!stricmp(token, "::") && !embed)
 			break;
 
-		if(!strnicmp(token, "exec.", 5) && embed)
+		if(!strnicmp(token, "exec.", 5) && (embed || apps))
 			execflag = true;
 
-		if(!stricmp(token, "exec") && embed)
+		if(!stricmp(token, "exec") && (embed || apps))
 			execflag = true;
 
 		if(!strnicmp(token, "exec.", 5) && !execflag && exec_token)
@@ -904,7 +908,7 @@ noexec:
 			snprintf(catchname, sizeof(catchname), "use.%s", token);
 			token = catchname;
 		}
-		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && !cmds->ripple)
+		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && !cmds->ripple && !apps && !embed)
 		{
 			token = getToken();
 			if(!token)
@@ -940,7 +944,7 @@ noexec:
 			pretoken = token + 3;
 			token = "call";
 		}
-		else if(strstr(token, "::") && !cmds->ripple)
+		else if(strstr(token, "::") && !cmds->ripple && !apps)
 		{
 			pretoken = token;
 			token = "call";
@@ -1084,7 +1088,7 @@ noexec:
 
 		addmask = submask = 0;
 
-		if(!cmds->ripple)
+		if(!cmds->ripple && !apps)
 		{
 			if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
 			{
@@ -1172,7 +1176,10 @@ noexec:
 
 	if(!scrStream->eof())
 	{
-		if(strstr(name, "::") || cmds->ripple)
+		execflag = false;
+		if(apps && apps_prefix)
+			snprintf(namebuf, sizeof(namebuf), "%s::%s", apps_prefix, name);
+		else if(strstr(name, "::") || cmds->ripple || apps)
 			snprintf(namebuf, sizeof(namebuf), "%s", name);
 		else
 			snprintf(namebuf, sizeof(namebuf), "%s::%s", basename, name);
